@@ -495,7 +495,7 @@ static void json_encode_array(smart_str *buf, zval **val, int options TSRMLS_DC)
 /* }}} */
 
 
-static int json_utf8_to_utf16(unsigned short *utf16, char utf8[], int len) /* {{{ */
+static int json_utf8_to_utf16(unsigned short *utf16, const char utf8[], int len) /* {{{ */
 {
 	size_t pos = 0, us;
 	int j, status;
@@ -829,14 +829,16 @@ static void json_object_to_zval(json_object  *new_obj, zval *return_value, int o
 				break;
 
 			case json_type_string:
-				RETVAL_STRINGL(json_object_get_string(new_obj), json_object_get_string_len(new_obj), 1);
+				{
+					const char *str = json_object_get_string(new_obj);
+					int         len = json_object_get_string_len(new_obj);
+
+					RETVAL_STRINGL(str, len, 1);
+				}
 				break;
 
 			case json_type_int:
 				i64 = json_object_get_int64(new_obj);
-				if (i64==INT64_MAX || i64==INT64_MIN) {
-					php_error_docref(NULL TSRMLS_CC, E_NOTICE, "integer overflow detected");
-				}
 #if SIZEOF_LONG > 4
 				RETVAL_LONG(i64);
 #else
@@ -886,6 +888,10 @@ static void json_object_to_zval(json_object  *new_obj, zval *return_value, int o
 				while (!json_object_iter_equal(&it, &itEnd)) {
 					MAKE_STD_ZVAL(tmpval);
 					key = json_object_iter_peek_name(&it);
+					if (!(options & PHP_JSON_PARSER_NOTSTRICT) &&
+						json_utf8_to_utf16(NULL, key, strlen(key)) < 0) {
+						JSON_G(error_code) = PHP_JSON_ERROR_UTF8;
+					}
 					tmpobj  = json_object_iter_peek_value(&it);
 					json_object_to_zval(tmpobj, tmpval, options TSRMLS_CC);
 
@@ -911,12 +917,18 @@ PHP_JSON_API void php_json_decode_ex(zval *return_value, char *str, int str_len,
 {
 	json_tokener *tok;
 	json_object  *new_obj;
-
+	int tok_opt = 0;
 	if (depth <= 0) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Depth must be greater than zero");
 		RETURN_NULL();
 	}
 
+	/* check UTF8 globally on input string, before parsing */
+	if (!(options & PHP_JSON_PARSER_NOTSTRICT) && json_utf8_to_utf16(NULL, str, str_len) < 0) {
+		JSON_G(error_code) = PHP_JSON_ERROR_UTF8;
+		RETURN_NULL();
+	}
+	JSON_G(error_code) = 0;
 	RETVAL_NULL();
 
 	tok = json_tokener_new_ex(depth);
@@ -924,8 +936,14 @@ PHP_JSON_API void php_json_decode_ex(zval *return_value, char *str, int str_len,
 		return;
 	}
 	if (!(options & PHP_JSON_PARSER_NOTSTRICT)) {
-		json_tokener_set_flags(tok, JSON_TOKENER_STRICT);
+		tok_opt = JSON_TOKENER_STRICT;
 	}
+	if (options & PHP_JSON_BIGINT_AS_STRING) {
+		tok_opt |= JSON_TOKENER_BIGINT_AS_STRING;
+	}
+	json_tokener_set_flags(tok, tok_opt);
+	json_tokener_set_bigint(tok, long_min_digits);
+
 	new_obj = json_tokener_parse_ex(tok, str, str_len);
 	if (json_tokener_get_error(tok)==json_tokener_continue) {
 		new_obj = json_tokener_parse_ex(tok, "", -1);
@@ -1064,7 +1082,7 @@ static PHP_FUNCTION(json_last_error_msg)
 */
 static PHP_METHOD(JsonIncrementalParser, __construct)
 {
-	long                           depth = JSON_PARSER_DEFAULT_DEPTH, options = 0;
+	long                           depth = JSON_PARSER_DEFAULT_DEPTH, options = 0, tok_opt=0;
 	struct php_json_object_parser *intern;
 	zend_error_handling            error_handling;
 
@@ -1083,8 +1101,13 @@ static PHP_METHOD(JsonIncrementalParser, __construct)
 		zend_throw_exception(zend_exception_get_default(TSRMLS_C), "Can't allocate parser", 0 TSRMLS_CC);
 	}
 	if (!(options & PHP_JSON_PARSER_NOTSTRICT)) {
-		json_tokener_set_flags(intern->tok, JSON_TOKENER_STRICT);
+		tok_opt = JSON_TOKENER_STRICT;
 	}
+	if (options & PHP_JSON_BIGINT_AS_STRING) {
+		tok_opt |= JSON_TOKENER_BIGINT_AS_STRING;
+	}
+	json_tokener_set_flags(intern->tok, tok_opt);
+	json_tokener_set_bigint(intern->tok, long_min_digits);
 
 }
 /* }}} */
